# cta.py
#
# Read the ctabus.csv using different data structures
import csv

def read_data(filename):
    records = []
    with open(filename) as f:
        rows = csv.reader(f)
        headers = next(rows)   # Skip the headers on first line
        for row in rows:
            records.append(make_record(row))
    return records

class Record(object):
    __slots__ = ('route', 'date', 'daytype', 'rides')
    def __init__(self, route, date, daytype, rides):
        self.route = route
        self.date = date
        self.daytype = daytype
        self.rides = rides

# Alternative: Namedtuple
from collections import namedtuple
Record = namedtuple('Record', ['route','date','daytype','rides'])

# Make a string cache of common strings (in the data)
_cache = { }

def cached(s):
    if s in _cache:
        return _cache[s]
    else:
        _cache[s] = s
        return s

from datetime import date

def make_record(row):
    # Represent a row as a Python data structure
    # Choice 1: Tuple.   184 Mbytes.
    # return (row[0], row[1], row[2], int(row[3]))

    # Choice 2: Dict. 300 Mbytes
##    return {
##        'route': row[0],
##        'date': row[1],
##        'daytype': row[2],
##        'rides': int(row[3]),
##        'spam': None,   # No difference in memory
##        'blah': None,   # 404 Mbytes
##        }

    # Choice 3: Use a class.  Worse or better than dicts?
    # 248 Mbytes
    # 177 Mbytes with slots
    # 189 Mbytes with namedtuple
    # 104 Mbytes with string caching.

    # Question: Is it better to leave the date as a string
    # or to convert it to date object (from datetime module)?

    parts = row[1].split('/')
    d = date(int(parts[2]), int(parts[0]), int(parts[1]))
    return Record(cached(row[0]), cached(d), row[2], int(row[3]))

records = read_data('ctabus.csv')
import os
print(os.getpid())

        
